'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql');

var _globalId = require('./globalId');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }
/* eslint-disable no-param-reassign */

var MutationMiddleware = function (_ResolverMiddleware) {
  _inherits(MutationMiddleware, _ResolverMiddleware);

  function MutationMiddleware(typeComposer, typeResolver) {
    var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    _classCallCheck(this, MutationMiddleware);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(MutationMiddleware).call(this, typeComposer, typeResolver, opts));

    _this.args = function (next) {
      return function (args) {
        var nextArgs = next(args);
        var inputTC = void 0;
        var newNextArgs = {};

        if (nextArgs.input && nextArgs.input.type) {
          // pass args unchanged
          inputTC = new _graphqlCompose.InputTypeComposer((0, _graphql.getNamedType)(nextArgs.input.type));
          newNextArgs = nextArgs;
        } else {
          // create input arg, and put into all current args
          inputTC = new _graphqlCompose.InputTypeComposer(new _graphql.GraphQLInputObjectType({
            name: 'Relay' + _this.typeResolver.getNameCamelCase() + _this.typeComposer.getTypeName() + 'Input',
            fields: nextArgs
          }));
          newNextArgs = {
            input: {
              name: 'input',
              type: new _graphql.GraphQLNonNull(inputTC.getType())
            }
          };
          _this.isArgsWrapped = true;
        }

        // add `clientMutationId` to args.input field
        if (!inputTC.hasField('clientMutationId')) {
          inputTC.addField('clientMutationId', {
            type: _graphql.GraphQLString,
            description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
          });
        }

        return newNextArgs;
      };
    };

    _this.resolve = function (next) {
      return function (resolveParams) {
        var clientMutationId = void 0;

        if (resolveParams && resolveParams.args) {
          if (resolveParams.args.input && resolveParams.args.input.clientMutationId) {
            clientMutationId = resolveParams.args.input.clientMutationId;
            delete resolveParams.args.input.clientMutationId;
          }
        }

        if (_this.isArgsWrapped) {
          // $FlowFixMe
          resolveParams.args = resolveParams.args.input;
        }

        return next(resolveParams).then(function (res) {
          res.nodeId = (0, _globalId.toGlobalId)(_this.typeComposer.getTypeName(), res.recordId);
          if (clientMutationId) {
            res.clientMutationId = clientMutationId;
          }
          return res;
        });
      };
    };

    _this.outputType = function (next) {
      return function (outputType) {
        var nextOutputType = next(outputType);

        if (!(nextOutputType instanceof _graphql.GraphQLObjectType)) {
          return nextOutputType;
        }

        var outputTC = new _graphqlCompose.TypeComposer(nextOutputType);
        if (!outputTC.hasField('nodeId')) {
          outputTC.addField('nodeId', {
            type: _graphql.GraphQLID,
            description: 'The globally unique ID among all types'
          });
        }

        if (!outputTC.hasField('clientMutationId')) {
          outputTC.addField('clientMutationId', {
            type: _graphql.GraphQLString,
            description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
          });
        }

        return outputTC.getType();
      };
    };

    _this.isArgsWrapped = false;
    return _this;
  }

  return MutationMiddleware;
}(_graphqlCompose.ResolverMiddleware);

exports.default = MutationMiddleware;